#include "utils.h"

#include <iomanip>
#include <numeric>

std::string formatCentAsEuroString(const int cent, int width)
{
    std::stringstream currStream;

    try {
        std::locale myLocale("de_DE.utf8");
        currStream.imbue(myLocale);
        currStream << std::right << std::setw(width) << std::showbase
                   << std::put_money(cent, false);
    } catch (std::runtime_error& err) {
        currStream << std::fixed << std::setw(width >= 4 ? width - 4 : width)
                   << std::setprecision(2) << cent / 100.0L << " €";
    }

    return currStream.str();
}

std::optional<PrinterDevice> convertToPosPrinterDevice(const std::string& device,
                                                       const std::string& endpoint)
{
    if (device.empty()) {
        return std::nullopt;
    }

    PrinterDevice printerDevice;
    std::string delimiter = ":";
    try {
        printerDevice.idVendor = std::stoi(device.substr(0, device.find(delimiter)), 0, 16);
        printerDevice.idProduct = std::stoi(device.substr(device.find(delimiter) + 1), 0, 16);
        if (endpoint.empty()) {
            printerDevice.endpoint = 0x03;
        } else {
            printerDevice.endpoint = std::stoi(endpoint, 0, 16);
        }

    } catch (std::exception& ex) {
        throw ex;
    }

    return printerDevice;
}

std::string& ltrim(std::string& str, const std::string& chars)
{
    str.erase(0, str.find_first_not_of(chars));
    return str;
}
 
std::string& rtrim(std::string& str, const std::string& chars)
{
    str.erase(str.find_last_not_of(chars) + 1);
    return str;
}
 
std::string& trim(std::string& str, const std::string& chars)
{
    return ltrim(rtrim(str, chars), chars);
}